"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getObjAbstract = void 0;
var _1 = __importDefault(require("licia/$"));
var startWith_1 = __importDefault(require("licia/startWith"));
var isObj_1 = __importDefault(require("licia/isObj"));
var uniqId_1 = __importDefault(require("licia/uniqId"));
var upperFirst_1 = __importDefault(require("licia/upperFirst"));
var toNum_1 = __importDefault(require("licia/toNum"));
var chunk_1 = __importDefault(require("licia/chunk"));
var each_1 = __importDefault(require("licia/each"));
var isNaN_1 = __importDefault(require("licia/isNaN"));
var isNum_1 = __importDefault(require("licia/isNum"));
var isBool_1 = __importDefault(require("licia/isBool"));
var isStr_1 = __importDefault(require("licia/isStr"));
var truncate_1 = __importDefault(require("licia/truncate"));
var keys_1 = __importDefault(require("licia/keys"));
var lowerCase_1 = __importDefault(require("licia/lowerCase"));
var naturalSort_1 = __importDefault(require("licia/naturalSort"));
var util_1 = require("./util");
var Component_1 = __importDefault(require("../share/Component"));
var JsonViewer = (function (_super) {
    __extends(JsonViewer, _super);
    function JsonViewer(container) {
        var _this = _super.call(this, container, { compName: 'object-viewer' }) || this;
        _this.onItemClick = function (e) {
            var _a = _this, map = _a.map, c = _a.c;
            var $this = (0, _1.default)(e.curTarget);
            var circularId = $this.data('object-id');
            var $firstSpan = $this.find('span').eq(0);
            if ($this.data('first-level'))
                return;
            if (circularId) {
                $this.find('ul').html(_this.objToHtml(map[circularId], false));
                $this.rmAttr('data-object-id');
            }
            e.stopImmediatePropagation();
            if (!$firstSpan.hasClass(c('expanded')))
                return;
            var $ul = $this.find('ul').eq(0);
            if ($firstSpan.hasClass(c('collapsed'))) {
                $firstSpan.rmClass(c('collapsed'));
                $ul.show();
            }
            else {
                $firstSpan.addClass(c('collapsed'));
                $ul.hide();
            }
            _this.emit('change');
        };
        _this.bindEvent();
        return _this;
    }
    JsonViewer.prototype.set = function (data) {
        if ((0, isStr_1.default)(data))
            data = JSON.parse(data);
        this.data = {
            id: (0, uniqId_1.default)('json'),
            enumerable: {
                0: data,
            },
        };
        this.map = {};
        createMap(this.map, this.data);
        this.render();
    };
    JsonViewer.prototype.destroy = function () {
        _super.prototype.destroy.call(this);
        this.$container.off('click', 'li', this.onItemClick);
    };
    JsonViewer.prototype.objToHtml = function (data, firstLevel) {
        var _this = this;
        var ret = '';
        (0, each_1.default)(['enumerable', 'unenumerable', 'symbol'], function (type) {
            if (!data[type])
                return;
            var typeKeys = (0, keys_1.default)(data[type]);
            (0, naturalSort_1.default)(typeKeys);
            for (var i = 0, len = typeKeys.length; i < len; i++) {
                var key = typeKeys[i];
                ret += _this.createEl(key, data[type][key], type, firstLevel);
            }
        });
        if (data.proto) {
            if (ret === '') {
                ret = this.objToHtml(data.proto);
            }
            else {
                ret += this.createEl('[[Prototype]]', data.proto, 'proto');
            }
        }
        return ret;
    };
    JsonViewer.prototype.createEl = function (key, val, keyType, firstLevel) {
        if (firstLevel === void 0) { firstLevel = false; }
        var c = this.c;
        var type = typeof val;
        if (val === null) {
            return "<li>".concat(wrapKey(key), "<span class=\"").concat(c('null'), "\">null</span></li>");
        }
        else if ((0, isNum_1.default)(val) || (0, isBool_1.default)(val)) {
            return "<li>".concat(wrapKey(key), "<span class=\"").concat(c(type), "\">").concat((0, util_1.encode)(val), "</span></li>");
        }
        if (val.type === 'RegExp')
            type = 'regexp';
        if (val.type === 'Number')
            type = 'number';
        if (val.type === 'Number' || val.type === 'RegExp') {
            return "<li>".concat(wrapKey(key), "<span class=\"").concat(c(type), "\">").concat((0, util_1.encode)(val.value), "</span></li>");
        }
        else if (val.type === 'Undefined' || val.type === 'Symbol') {
            return "<li>".concat(wrapKey(key), "<span class=\"").concat(c('special'), "\">").concat((0, lowerCase_1.default)(val.type), "</span></li>");
        }
        else if (val === '(...)') {
            return "<li>".concat(wrapKey(key), "<span class=\"").concat(c('special'), "\">").concat(val, "</span></li>");
        }
        else if ((0, isObj_1.default)(val)) {
            var id = val.id;
            var referenceId = val.reference;
            var objAbstract = getObjAbstract(val) || (0, upperFirst_1.default)(type);
            var icon = firstLevel
                ? ''
                : "<span class=\"".concat(c('expanded collapsed'), "\"><span class=\"").concat(c('icon icon-caret-right'), "\"></span><span class=\"").concat(c('icon icon-caret-down'), "\"></span></span>");
            var obj = "<li ".concat(firstLevel ? 'data-first-level="true"' : '', " ").concat('data-object-id="' + (referenceId || id) + '"', ">").concat(icon).concat(wrapKey(key), "<span class=\"").concat(c('open'), "\">").concat(firstLevel ? '' : objAbstract, "</span><ul class=\"").concat(c(type), "\" ").concat(firstLevel ? '' : 'style="display:none"', ">");
            if (firstLevel)
                obj += this.objToHtml(this.map[id]);
            return obj + "</ul><span class=\"".concat(c('close'), "\"></span></li>");
        }
        function wrapKey(key) {
            if (firstLevel)
                return '';
            if ((0, isObj_1.default)(val) && val.jsonSplitArr)
                return '';
            var keyClass = c('key');
            if (keyType === 'unenumerable' || keyType === 'symbol') {
                keyClass = c('key-lighter');
            }
            else if (keyType === 'proto') {
                keyClass = c('key-special');
            }
            return "<span class=\"".concat(keyClass, "\">").concat((0, util_1.encode)(key), "</span>: ");
        }
        if ((0, isStr_1.default)(val) && val.length > 10000) {
            val = (0, truncate_1.default)(val, 50, {
                separator: ' ',
                ellipsis: '…',
            });
        }
        return "<li>".concat(wrapKey(key), "<span class=\"").concat(c(typeof val), "\">\"").concat((0, util_1.encode)(val), "\"</span></li>");
    };
    JsonViewer.prototype.render = function () {
        var data = this.map[this.data.id];
        this.$container.html(this.objToHtml(data, true));
    };
    JsonViewer.prototype.bindEvent = function () {
        this.$container.on('click', 'li', this.onItemClick);
    };
    return JsonViewer;
}(Component_1.default));
exports.default = JsonViewer;
function createMap(map, data) {
    var id = data.id;
    if (!id && id !== 0)
        return;
    var isArr = data.type && (0, startWith_1.default)(data.type, 'Array');
    if (isArr && data.enumerable) {
        var arr = objToArr(data, id, data.type);
        if (arr.length > 100)
            data = splitBigArr(arr);
    }
    map[id] = data;
    var values = [];
    (0, each_1.default)(['enumerable', 'unenumerable', 'symbol'], function (type) {
        if (!data[type])
            return;
        for (var key in data[type]) {
            values.push(data[type][key]);
        }
    });
    if (data.proto) {
        values.push(data.proto);
    }
    for (var i = 0, len = values.length; i < len; i++) {
        var val = values[i];
        if ((0, isObj_1.default)(val))
            createMap(map, val);
    }
}
function splitBigArr(data) {
    var idx = 0;
    var enumerable = {};
    (0, each_1.default)((0, chunk_1.default)(data, 100), function (val) {
        var obj = {};
        var startIdx = idx;
        obj.type = '[' + startIdx;
        obj.enumerable = {};
        (0, each_1.default)(val, function (val) {
            obj.enumerable[idx] = val;
            idx += 1;
        });
        var endIdx = idx - 1;
        obj.type += (endIdx - startIdx > 0 ? ' … ' + endIdx : '') + ']';
        obj.id = (0, uniqId_1.default)('json');
        obj.jsonSplitArr = true;
        enumerable[idx] = obj;
    });
    var ret = {};
    ret.enumerable = enumerable;
    ret.id = data.id;
    ret.type = data.type;
    if (data.unenumerable)
        ret.unenumerable = data.unenumerable;
    if (data.symbol)
        ret.symbol = data.symbol;
    if (data.proto)
        ret.proto = data.proto;
    return ret;
}
function objToArr(data, id, type) {
    var ret = [];
    var enumerable = {};
    (0, each_1.default)(data.enumerable, function (val, key) {
        var idx = (0, toNum_1.default)(key);
        if (!(0, isNaN_1.default)(idx)) {
            ret[idx] = val;
        }
        else {
            enumerable[key] = val;
        }
    });
    ret.enumerable = enumerable;
    ret.type = type;
    ret.id = id;
    if (data.unenumerable)
        ret.unenumerable = data.unenumerable;
    if (data.symbol)
        ret.symbol = data.symbol;
    if (data.proto)
        ret.proto = data.proto;
    return ret;
}
function getObjAbstract(data) {
    var type = data.type, value = data.value;
    if (!type)
        return;
    if (type === 'Function') {
        return (0, util_1.getFnAbstract)(value);
    }
    if (type === 'Array' && data.unenumerable) {
        return "Array(".concat(data.unenumerable.length, ")");
    }
    return data.type;
}
exports.getObjAbstract = getObjAbstract;
